import argparse
import ast
import sys

import chardet
import json
import os
import gitignore_parser


def describe_directory(directory, ignore_git=True):
    """
    Recursively describe all files in a directory.
    """
    gitignore_path = os.path.abspath(os.path.join(directory, '.gitignore'))
    gitignore = gitignore_parser.parse_gitignore(gitignore_path) if ignore_git and os.path.isfile(
        gitignore_path) else None
    output = {}
    for root, dirs, files in os.walk(directory):
        for file in files:
            path = os.path.join(root, file)
            if file.endswith('.py') and (gitignore is None or not gitignore(path)):
                with open(path, 'rb') as f:
                    content = f.read()
                    encoding = chardet.detect(content)['encoding']
                with open(path, encoding=encoding) as f:
                    print(path)
                    code = f.read()
                    module = ast.parse(code)
                    module_data = {}
                    functions = {}
                    classes = {}
                    for node in module.body:
                        if isinstance(node, ast.FunctionDef):
                            signature = ast.unparse(node.args)
                            if ast.get_docstring(node) != None:
                                docstring = _clean_output(ast.get_docstring(node))
                            functions.update(
                                {node.name.replace(" ", "") + "(" + signature.replace(" ", "") + ")": docstring})
                        elif isinstance(node, ast.ClassDef):
                            if ast.get_docstring(node) != None:
                                docstring = _clean_output(ast.get_docstring(node))
                            methods = {}
                            for method in node.body:
                                if isinstance(method, ast.FunctionDef):
                                    signature = ast.unparse(method.args)
                                    if ast.get_docstring(method) != None:
                                        docstring = _clean_output(ast.get_docstring(method))
                                    methods.update({method.name.replace(" ", "") + "(" + signature.replace(" ",
                                               "") + ")": docstring})
                            classes.update({node.name.replace(" ", ""): {"methods": methods}})
                    if functions:
                        module_data["functions"] = functions
                    if classes:
                        module_data["classes"] = classes
                    if module_data:
                        output[path.replace(" ", "")] = module_data

    # add argparse help text to output
    parser = argparse.ArgumentParser(description='Describe Python directory')
    help_text = parser.format_help()
    output['argparse_help'] = _clean_output(help_text)

    return output

def _clean_output(output):
    """
    Removes extra spaces, tabs, and newlines from argparse output.
    """
    output = output.replace('\n', ' ')
    output = output.replace('\t', '')
    output = ' '.join(output.split())
    return output




def main():
    """
    Parse command line arguments and describe directory.
    """
    if len(sys.argv) < 2:
        print("Usage: pydescribe.py <directory>")
        sys.exit(1)
    directory = sys.argv[1]
    ignore_git = True
    if len(sys.argv) > 2 and sys.argv[2] == '--no-gitignore':
        ignore_git = False
    output = describe_directory(directory, ignore_git=ignore_git)
    print(json.dumps(output, indent=4))


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['inspect_dense']

package_data = \
{'': ['*']}

install_requires = \
['gitignore-parser>=0.1.3,<0.2.0']

entry_points = \
{'console_scripts': ['inspect-dense = inspect_dense.getcode:main']}

setup_kwargs = {
    'name': 'inspect-dense',
    'version': '0.1.98',
    'description': 'A tool for recursively describing all files in a directory.',
    'long_description': 'inspect-dense\n==========\n\ninspect-dense is a Python package for recursively describing all python files in a directory. \nProvides a simple way to get a high-level overview of a project\'s codebase in a conscise format to minimize tokens used.\n\nInstallation\n------------\n\nYou can install inspect-dense using pip:\n\nCopy code\n\n`pip install inspect-dense`\n\nUsage\n-----\n\nTo use inspect-dense, simply import the `describe_directory` function from the package and pass it the directory path you want to describe:\n\npython code\n```python\nfrom inspect-dense import describe_directory  \n\ndirectory = "/path/to/directory" \noutput = describe_directory(directory)\n```\n\nThe `describe_directory` function returns a dictionary with the following structure:\n\n```json\n{     "/path/to/file.py": {         "functions": {             "function_name(args)": "Function docstring"         },         "classes": {             "Class1": {                 "methods": {                     "method1(args)": "Method docstring",                     "method2(args)": "Method docstring"                 }             },             "Class2": {                 "methods": {                     "method3(args)": "Method docstring",                     "method4(args)": "Method docstring"                 }             }         }     } }\n```\nCommand Line Interface\n----------------------\n\ninspect-dense also comes with a command line interface. You can use it to describe a directory and output the results to a file:\n\n`inspect-dense <directory> [--no-gitignore]`\n\nFor example:\n\n`inspect-dense /path/to/directory --no-gitignore`\n\nThis will output the results.\n\nLicense\n-------\n\nThis project is licensed under the Apache 2 License - see the [LICENSE](LICENSE) file for details.',
    'author': 'Justin Riddiough',
    'author_email': 'justin@visioninit.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/neural-loop/inspect_dense',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
